@@ -49,6 +49,8 @@ char *progname;
 
 char *smtp_host = "localhost";
 int smtp_port = 24;
+char *rcfile_name = NULL;
+char *netrc_name = NULL;
 struct obstack input_stk;
 
 int verbose;
@@ -69,6 +71,7 @@ void error (const char *, ...);
 int send_line (char *buf);
 int smtp_get_reply (struct smtp_reply *repl);
 void smtp_free_reply (struct smtp_reply *repl);
+static void smtp_quit (void);
 
 #define R_CONT     0x8000
 #define R_CODEMASK 0xfff
@@ -107,13 +110,15 @@ info (int mode, const char *fmt, ...)
 }
 
 void
-anubis_error (int ignored_method, const char *fmt, ...)
+anubis_error (int exit_code, int error_code, const char *fmt, ...)
 {
   va_list ap;
 
   va_start (ap, fmt);
   fprintf (stderr, "%s: ", progname);
   vfprintf (stderr, fmt, ap);
+  if (error_code)
+    fprintf (stderr, ": %s", strerror (error_code));
   fprintf (stderr, "\n");
   va_end (ap);
 }
@@ -135,6 +140,7 @@ starttls (void)
   if (!iostream)
     {
       error (_("TLS negotiation failed"));
+      smtp_quit ();
       exit (1);
     }
 }
@@ -434,11 +440,11 @@ smtp_print_reply (FILE * fp, struct smtp_reply *repl)
 
 
 void
-smtp_ehlo (void)
+smtp_ehlo (int xelo)
 {
   struct smtp_reply repl;
 
-  send_line ("EHLO localhost");
+  send_line (xelo ? "XELO localhost" : "EHLO localhost");
   smtp_get_reply (&repl);
   if (repl.code != 250)
     {
@@ -900,6 +906,7 @@ do_gsasl_auth (Gsasl_ctx * ctx, char *mech)
     {
       error (_("Authentication failed"));
       smtp_print_reply (stderr, &repl);
+      smtp_quit ();
       exit (1);
     }
 
@@ -974,8 +981,12 @@ char *
 rc_name (void)
 {
   char *rc;
-  const char *home = get_home_dir ();
+  const char *home;
 
+  if (rcfile_name)
+    return rcfile_name;
+  
+  home = get_home_dir ();
   rc = xmalloc (strlen (home) + 1 + sizeof DEFAULT_LOCAL_RCFILE);
   strcpy (rc, home);
   strcat (rc, "/");
@@ -988,103 +999,30 @@ rc_name (void)
 #define CMP_ERROR     2
 
 int
-diff (char *file1, char *file2)
+diff (char *file, struct smtp_reply *repl)
 {
-  pid_t pid;
-  int i, status;
-  char *args[4];
-
-  VDETAIL (1, (_("Comparing %s and %s\n"), file1, file2));
-
-  args[0] = "cmp";
-  args[1] = file1;
-  args[2] = file2;
-  args[3] = NULL;
-
-  switch (pid = fork ())
-    {
-    case -1:			/* an error */
-      error (_("fork() failed: %s"), strerror (errno));
-      return -1;
-
-    case 0:			/* a child process */
-      for (i = 0; i < 64; i++)	/* FIXME */
-	close (i);
-      open ("/dev/null", O_RDONLY);
-      open ("/dev/null", O_WRONLY);
-      execvp (args[0], args);
-      error (_("execvp() failed: %s"), strerror (errno));
-      exit (77);
-    }
-
-  /* Master */
-  waitpid (pid, &status, 0);
-  if (WIFEXITED (status))
-    {
-      switch (WEXITSTATUS (status)) {
-      case 0:
-	VDETAIL (0, (_("Result: UNCHANGED\n")));
-	return CMP_UNCHANGED;
-      case 1:
-	VDETAIL (0, (_("Result: CHANGED\n")));
-	return CMP_CHANGED;
-      default:
-	VDETAIL (0, (_("Result: ERROR\n")));
-	return CMP_ERROR;
-      }
-    }
-  VDETAIL (1, (_("Result: Abnormal termination\n")));
-
-  return CMP_ERROR;
-}
-
-#ifndef P_tmpdir
-# define P_tmpdir "/tmp"
-#endif
-#define TEMPLATE "anXXXXXX"
-
-char *
-save_reply (struct smtp_reply *reply)
-{
-  char *filename;
-  int fd;
-  int i;
-  char *tmpdir = getenv ("TMPDIR");
-
-  if (!tmpdir)
-    tmpdir = P_tmpdir;
-
-  filename = xmalloc (strlen (tmpdir) + 1 + sizeof TEMPLATE);
-  sprintf (filename, "%s/%s", tmpdir, TEMPLATE);
-
-#ifdef HAVE_MKSTEMP
-  {
-    int save_mask = umask (077);
-    fd = mkstemp (filename);
-    umask (save_mask);
-  }
-#else
-  if (mktemp (filename))
-    fd = open (filename, O_CREAT | O_EXCL | O_RDWR, 0600);
-  else
-    fd = -1;
-#endif
-
+  unsigned char sample[MD5_DIGEST_BYTES];
+  unsigned char digest[MD5_DIGEST_BYTES];
+  int len;
+  int fd = open (file, O_RDONLY);
   if (fd == -1)
     {
-      error (_("Can not open temporary file: %s"), strerror (errno));
-      free (filename);
-      exit (1);
-    }
-
-  VDETAIL (1, (_("Saving to %s\n"), filename));
-  for (i = 1; i < reply->argc - 1; i++)
-    {
-      write (fd, reply->argv[i], strlen (reply->argv[i]));
-      write (fd, "\n", 1);
+      error (_("Cannot open file %s: %s"), file, strerror (errno));
+      return CMP_ERROR;
     }
+  anubis_md5_file (digest, fd);
   close (fd);
-  return filename;
+
+  len = strlen (repl->argv[0]);
+  if (len != sizeof digest * 2)
+    {
+      error (_("Invalid MD5 digest: %s"), repl->argv[0]);
+      return CMP_ERROR;
+    }
+  string_hex_to_bin (sample, repl->argv[0], len);
+
+  return memcmp (digest, sample, sizeof digest) == 0 ?
+                 CMP_UNCHANGED : CMP_CHANGED;
 }
 
 void
@@ -1144,7 +1082,6 @@ synch (void)
   int rc;
   struct sockaddr_in addr;
   struct smtp_reply repl;
-  char *filename;
   char *rcname;
 
   obstack_init (&input_stk);
@@ -1181,20 +1118,19 @@ synch (void)
     }
   smtp_free_reply (&repl);
 
-  smtp_ehlo ();
+  smtp_ehlo (1);
 
 #ifdef HAVE_TLS
   if (enable_tls && find_capa (&smtp_capa, "STARTTLS", NULL) == 0)
     {
       starttls ();
-      smtp_ehlo ();
+      smtp_ehlo (0);
     }
 #endif
 
   smtp_auth ();
-
   /* Get the capabilities */
-  smtp_ehlo ();
+  smtp_ehlo (0);
 
   if (find_capa (&smtp_capa, "XDATABASE", NULL))
     {
@@ -1203,27 +1139,35 @@ synch (void)
       return 1;
     }
 
+  
   send_line ("XDATABASE EXAMINE");
   smtp_get_reply (&repl);
-  if (repl.code != 250)
+  switch (repl.code)
     {
+    case 300:
+      rcname = rc_name ();
+      rc = CMP_CHANGED;
+      break;
+	
+    case 250:
+      rcname = rc_name ();
+      rc = diff (rcname, &repl);
+      break;
+      
+    default:
       error (_("EXAMINE failed"));
       smtp_print_reply (stderr, &repl);
       smtp_quit ();
       return 1;
     }
 
-  filename = save_reply (&repl);
-  rcname = rc_name ();
-
-  rc = diff (filename, rcname);
-
-  unlink (filename);
-  free (filename);
-
   if (rc == CMP_CHANGED)
-    smtp_upload (rcname);
-  free (rcname);
+    {
+      VDETAIL (1, (_("File changed\n")));
+      smtp_upload (rcname);
+    }
+  else
+    VDETAIL (1, (_("File NOT changed\n")));
 
   smtp_quit ();
   return 0;
@@ -1238,6 +1182,8 @@ static struct option gnu_options[] = {
   {"verbose", no_argument, 0, 'v'},
   {"version", no_argument, 0, OPT_VERSION},
   {"help", no_argument, 0, OPT_HELP},
+  {"file", required_argument, 0, 'f'},
+  {"netrc", required_argument, 0, 'n'},
 #ifdef HAVE_TLS
   {"disable-tls", no_argument, 0, 'd'},
   {"tls-cafile", required_argument, 0, 'C'},
@@ -1256,7 +1202,9 @@ help (void)
   puts (_("  -d, --disable-tls       Disable TLS encryption"));
   puts (_("  -C, --tls-cafile FILE   Use given CA file"));
 #endif
+  puts (_("  -f, --file FILE         Set user configuration file name"));
   puts (_("  -m, --mechanism MECH    Restrict allowed SASL mechanisms"));
+  puts (_("  -n, --netrc FILE        Set .netrc file name"));
   puts (_("  -v, --verbose           Verbose output. Multiple options\n"
 	  "                          increase the verbosity. Maximum is\n"
 	  "                          3"));
@@ -1271,13 +1219,18 @@ help (void)
 void
 read_netrc (void)
 {
-  const char *home = get_home_dir ();
-  char *netrc = xmalloc (strlen (home) + 1 + sizeof NETRC_NAME);
-  strcpy (netrc, home);
-  strcat (netrc, "/");
-  strcat (netrc, NETRC_NAME);
-  parse_netrc (netrc);
-  free (netrc);
+  if (netrc_name)
+    parse_netrc (netrc_name);
+  else
+    {
+      const char *home = get_home_dir ();
+      char *netrc = xmalloc (strlen (home) + 1 + sizeof NETRC_NAME);
+      strcpy (netrc, home);
+      strcat (netrc, "/");
+      strcat (netrc, NETRC_NAME);
+      parse_netrc (netrc);
+      free (netrc);
+    }
 }
 
 
@@ -1292,7 +1245,7 @@ main (int argc, char **argv)
   else
     progname++;
 
-  while ((c = getopt_long (argc, argv, "dC:m:v", gnu_options, NULL)) != EOF)
+  while ((c = getopt_long (argc, argv, "dC:f:n:m:v", gnu_options, NULL)) != EOF)
     {
       switch (c)
 	{
@@ -1306,6 +1259,14 @@ main (int argc, char **argv)
 	  break;
 
 #endif
+	case 'f':
+	  rcfile_name = optarg;
+	  break;
+
+	case 'n':
+	  netrc_name = optarg;
+	  break;
+	  
 	case 'm':
 	  add_mech (optarg);
 	  break;
@@ -1323,6 +1284,7 @@ main (int argc, char **argv)
 	  break;
 
 	default:
+	  fprintf (stderr, "%s: unknown option -%c\n", progname, optopt);
 	  exit (1);
 	}
     }
@@ -1361,3 +1323,4 @@ main (int argc, char **argv)
 }
 
 /* EOF */
+
