@@ -55,7 +55,7 @@ enum asmtp_state
   state_init,
   state_ehlo,
   state_auth,
-  state_quit
+  state_quit,
 };
 
 #define KW_HELO      0
@@ -67,6 +67,7 @@ enum asmtp_state
 #define KW_STARTTLS  6
 #define KW_MAIL      7
 #define KW_RCPT      8
+#define KW_XDATABASE 9
 
 static int
 asmtp_kw (const char *name)
@@ -76,8 +77,7 @@ asmtp_kw (const char *name)
     char *name;
     int code;
   }
-  kw[] =
-  {
+  kw[] = {
     { "helo", KW_HELO },
     { "ehlo", KW_EHLO },
     { "xelo", KW_XELO },
@@ -87,6 +87,7 @@ asmtp_kw (const char *name)
     { "starttls", KW_STARTTLS },
     { "mail", KW_MAIL },
     { "rcpt", KW_RCPT },
+    { "xdatabase", KW_XDATABASE },
     { NULL },
   };
   int i;
@@ -174,12 +175,10 @@ static void
 asmtp_capa_init (void)
 {
   asmtp_capa = list_create ();
-#ifdef HAVE_TLS
+#ifdef USE_SSL
   asmtp_capa_add ("STARTTLS");
 #endif
-#ifdef WITH_GSASL
   auth_gsasl_init ();
-#endif
   asmtp_capa_add ("HELP");
 }
 
@@ -234,7 +233,9 @@ asmtp_ehlo_reply (char *args)
       asmtp_reply (501, "Syntax error");
       return state_init;
     }
-
+  
+  set_ehlo_domain(domain);
+  
   helo_count++;
   asmtp_reply (R_CONT | 250, "Anubis is pleased to meet you.");
   asmtp_capa_report ();
@@ -310,6 +311,7 @@ asmtp_init (enum asmtp_state state)
     break;
     
   case KW_QUIT:
+    asmtp_reply (221, "Closing connection");
     state = state_quit;
     break;
     
@@ -347,6 +349,8 @@ asmtp_ehlo (enum asmtp_state state, ANUBIS_USER * usr)
       asmtp_reply(503, "TLS already started");
     else
       {
+	NET_STREAM stream;
+	
 	if (!secure.cert)
 	  secure.cert = allocbuf (DEFAULT_SSL_PEM, MAXPATHLEN);
 	if (!check_filename (secure.cert, NULL))
@@ -364,17 +368,17 @@ asmtp_ehlo (enum asmtp_state state, ANUBIS_USER * usr)
 	  }
 
 	asmtp_reply (220, "Ready to start TLS");
-	secure.server = start_ssl_server (remote_client,
-					  secure.cafile,
-					  secure.cert,
-					  secure.key,
-					  options.termlevel > NORMAL);
-	if (!secure.server || (topt & T_ERROR))
+	stream = start_ssl_server (remote_client,
+				   secure.cafile,
+				   secure.cert,
+				   secure.key,
+				   options.termlevel > NORMAL);
+	if (!stream)
 	  {
 	    asmtp_reply (454, "TLS not available" CRLF);
 	    break;
 	  }
-	remote_client = secure.server;
+	remote_client = stream;
 	asmtp_capa_remove ("STARTTLS");
 	topt |= T_SSL_FINISHED;
     
@@ -391,6 +395,7 @@ asmtp_ehlo (enum asmtp_state state, ANUBIS_USER * usr)
     break;
     
   case KW_QUIT:
+    asmtp_reply (221, "Closing connection");
     state = state_quit;
     break;
     
@@ -438,10 +443,66 @@ anubis_smtp (ANUBIS_USER * usr)
       }
     }
 
-  topt &= ~T_SSL_FINISHED;
+  if (topt & T_SSL_FINISHED)
+    {
+      /* If `ssl yes' is requested, convert it to `ssl-oneway' for
+	 the mechanics of tunnel.c:handle_ehlo() to work properly. */
+	 
+      topt &= ~T_SSL_FINISHED;
+      if (topt & T_SSL)
+	{
+	  topt &= ~T_SSL;
+	  topt |= T_SSL_ONEWAY;
+	}
+    }
   xdatabase_enable ();
+  
   return 0;
 }
+
+static void
+xdb_loop (void)
+{
+  char *command = NULL;
+  size_t s = 0;
+
+  info (VERBOSE, _("Entering XDB loop..."));
+
+  asmtp_capa_add ("XDATABASE");
+  while (recvline_ptr (SERVER, remote_client, &command, &s) > 0)
+    {
+      switch (asmtp_kw (get_command_word (command))) {
+      case KW_HELP:
+	asmtp_help ();
+	break;
+    
+      case KW_QUIT:
+	asmtp_reply (221, "Closing connection");
+	info (VERBOSE, _("Exiting XDB loop..."));
+	return;
+
+      case KW_XDATABASE:
+	xdatabase (make_lowercase (get_command_arg ()));
+	break;
+
+      case KW_EHLO:
+	asmtp_ehlo_reply (command);
+	break;
+	
+      case KW_AUTH:      
+      case KW_STARTTLS:  
+      case KW_MAIL:
+      case KW_RCPT:
+	asmtp_reply (550, "Command disabled.");
+	break;
+    
+      default:
+	asmtp_reply (500, "Unknown command");
+      }
+    }
+  info (VERBOSE, _("Exiting XDB loop..."));
+}
+
 
 
 static char *anubis_dbarg;
@@ -462,14 +523,14 @@ anubis_get_db_record (const char *username, ANUBIS_USER * usr)
 
   if (!anubis_dbarg)
     {
-      anubis_error (HARD, _("Database not specified"));
+      anubis_error (0, 0, _("Database not specified"));
       return ANUBIS_DB_FAIL;
     }
 
   if (anubis_db_open (anubis_dbarg, anubis_db_rdonly,
 		      &db, &errtext) != ANUBIS_DB_SUCCESS)
     {
-      anubis_error (HARD,
+      anubis_error (0, 0,
 		    _("Cannot open database %s: %s"), anubis_dbarg, errtext);
       return ANUBIS_DB_FAIL;
     }
@@ -481,7 +542,7 @@ anubis_get_db_record (const char *username, ANUBIS_USER * usr)
     break;
     
   case ANUBIS_DB_FAIL:
-    anubis_error (SOFT,
+    anubis_error (0, 0,
 		  _("Cannot retrieve data from the SASL database: %s"),
 		  anubis_db_strerror (db));
     break;
@@ -509,18 +570,11 @@ anubis_authenticate_mode (NET_STREAM *psd_client,
   if (anubis_smtp (&usr))
     return EXIT_FAILURE;
 
-  if (usr.rcfile_name)
-    session.rcfile_name = usr.rcfile_name;
-
   if (usr.username)
     {
       strncpy (session.clientname, usr.username, sizeof (session.clientname));
       if (check_username (session.clientname))
-	{
-	  anubis_changeowner (session.clientname);
-	  open_rcfile (CF_CLIENT);
-	  process_rcfile (CF_CLIENT);
-	}
+	anubis_changeowner (session.clientname);
       else
 	set_unprivileged_user ();
     }
@@ -531,102 +585,113 @@ anubis_authenticate_mode (NET_STREAM *psd_client,
       set_unprivileged_user ();
     }
 
-  if (!(topt & T_LOCAL_MTA) && strlen (session.mta) == 0)
+  if (usr.rcfile_name)
     {
-      anubis_error (HARD, _("MTA has not been specified. "
-			    "Set either REMOTE-MTA or LOCAL-MTA."));
-      return EXIT_FAILURE;
+      session.rcfile_name = usr.rcfile_name;
+      open_rcfile (CF_CLIENT);
+      process_rcfile (CF_CLIENT);
     }
-
-  /*
-     Protection against a loop connection.
-   */
-
-  if (!(topt & T_LOCAL_MTA))
+  
+  if (topt & T_XELO)
     {
-      unsigned long inaddr;
-      struct sockaddr_in ad;
-
-      memset (&ad, 0, sizeof (ad));
-      inaddr = inet_addr (session.mta);
-      if (inaddr != INADDR_NONE)
-	memcpy (&ad.sin_addr, &inaddr, sizeof (inaddr));
-      else
+      xdb_loop ();
+    }
+  else
+    {
+      if (!(topt & T_LOCAL_MTA) && strlen (session.mta) == 0)
 	{
-	  struct hostent *hp = 0;
-	  hp = gethostbyname (session.mta);
-	  if (hp == 0)
-	    {
-	      hostname_error (session.mta);
-	      return EXIT_FAILURE;
-	    }
+	  anubis_error (EXIT_FAILURE, 0,
+                        _("MTA has not been specified. "
+		  	  "Set either REMOTE-MTA or LOCAL-MTA."));
+	}
+
+      /*
+	Protection against a loop connection.
+      */
+      
+      if (!(topt & T_LOCAL_MTA))
+	{
+	  unsigned long inaddr;
+	  struct sockaddr_in ad;
+	  
+	  memset (&ad, 0, sizeof (ad));
+	  inaddr = inet_addr (session.mta);
+	  if (inaddr != INADDR_NONE)
+	    memcpy (&ad.sin_addr, &inaddr, sizeof (inaddr));
 	  else
 	    {
-	      if (hp->h_length != 4 && hp->h_length != 8)
+	      struct hostent *hp = 0;
+	      hp = gethostbyname (session.mta);
+	      if (hp == 0)
 		{
-		  anubis_error (HARD,
-				_
-				("Illegal address length received for host %s"),
-				session.mta);
+		  hostname_error (session.mta);
 		  return EXIT_FAILURE;
 		}
 	      else
 		{
-		  memcpy ((char *) &ad.sin_addr.s_addr,
-			  hp->h_addr, hp->h_length);
+		  if (hp->h_length != 4 && hp->h_length != 8)
+		    {
+		      anubis_error (EXIT_FAILURE, 0,
+			 _("Illegal address length received for host %s"),
+				    session.mta);
+		    }
+		  else
+		    {
+		      memcpy ((char *) &ad.sin_addr.s_addr,
+			      hp->h_addr, hp->h_length);
+		    }
 		}
 	    }
+	  if (ntohl (ad.sin_addr.s_addr) == INADDR_LOOPBACK
+	      && session.anubis_port == session.mta_port)
+	    {
+	      anubis_error (EXIT_FAILURE, 0, 
+                             _("Loop not allowed. Connection rejected."));
+	    }
 	}
-      if (ntohl (ad.sin_addr.s_addr) == INADDR_LOOPBACK
-	  && session.anubis_port == session.mta_port)
+      
+      alarm (300);
+      if (topt & T_LOCAL_MTA)
 	{
-	  anubis_error (SOFT, _("Loop not allowed. Connection rejected."));
-	  return EXIT_FAILURE;
+	  remote_server = make_local_connection (session.execpath,
+						 session.execargs);
+	  if (!remote_server)
+	    {
+	      service_unavailable (&remote_client);
+	      return EXIT_FAILURE;
+	    }
 	}
-    }
-
-  alarm (300);
-  if (topt & T_LOCAL_MTA)
-    {
-      remote_server = make_local_connection (session.execpath,
-					     session.execargs);
-      if (!remote_server)
+      else
 	{
-	  service_unavailable (&remote_client);
-	  return EXIT_FAILURE;
+	  remote_server = make_remote_connection (session.mta,
+						  session.mta_port);
+	  if (!remote_server)
+	    service_unavailable (&remote_client);
 	}
-    }
-  else
-    {
-      remote_server = make_remote_connection (session.mta, session.mta_port);
-      if (!remote_server)
-	service_unavailable (&remote_client);
-    }
-  alarm (0);
 
-  if (!(topt & T_ERROR))
-    {
       alarm (900);
       smtp_session ();
       alarm (0);
     }
+  
   net_close_stream (&remote_client);
   net_close_stream (&remote_server);
-
-  if (topt & T_ERROR)
-    info (NORMAL, _("Connection terminated."));
-  else
-    info (NORMAL, _("Connection closed successfully."));
+  *psd_client = NULL;
+  
+  info (NORMAL, _("Connection closed successfully."));
 
 #ifdef HAVE_PAM
-  pam_retval = pam_close_session (pamh, 0);
-  if (pam_retval == PAM_SUCCESS)
-    info (VERBOSE, _("PAM: Session closed."));
-  if (pam_end (pamh, pam_retval) != PAM_SUCCESS)
+  if (pamh)
     {
-      pamh = NULL;
-      info (NORMAL, _("PAM: failed to release authenticator."));
-      return EXIT_FAILURE;
+      int pam_retval = pam_close_session (pamh, 0);
+      if (pam_retval == PAM_SUCCESS)
+	info (VERBOSE, _("PAM: Session closed."));
+      if (pam_end (pamh, pam_retval) != PAM_SUCCESS)
+	{
+	  pamh = NULL;
+	  info (NORMAL, _("PAM: failed to release authenticator."));
+	  return EXIT_FAILURE;
+	}
     }
 #endif /* HAVE_PAM */
   return 0;
