@@ -43,16 +43,17 @@ static int nchild;
 void
 daemonize (void)
 {
+  signal (SIGHUP, SIG_IGN);
 #ifdef HAVE_DAEMON
   if (daemon (0, 0) == -1)
-    anubis_error (HARD, _("daemon() failed. %s."), strerror (errno));
+    anubis_error (EXIT_FAILURE, errno, _("daemon() failed"));
 #else
   chdir ("/");
   umask (0);
   switch (fork ())
     {
     case -1:			/* fork() failed */
-      anubis_error (HARD, _("Can't fork. %s."), strerror (errno));
+      anubis_error (EXIT_FAILURE, errno, _("Cannot fork."));
       break;
     case 0:			/* child process */
       break;
@@ -60,41 +61,69 @@ daemonize (void)
       quit (0);
     }
   if (setsid () == -1)
-    anubis_error (HARD, _("setsid() failed."));
+    anubis_error (EXIT_FAILURE, errno, _("setsid() failed"));
 
   close (0);
   close (1);
   close (2);
 #endif /* HAVE_DAEMON */
 
-  signal (SIGHUP, SIG_IGN);
   topt &= ~T_FOREGROUND;
   topt |= T_DAEMON;
 
 #ifdef HAVE_SYSLOG
   openlog ("anubis", LOG_PID, 0);
-  syslog (LOG_INFO, _("%s daemon startup succeeded."), version);
+  info (NORMAL, _("%s daemon startup succeeded."), version);
 #endif /* HAVE_SYSLOG */
-
+  write_pid_file ();
   return;
 }
 
+char *
+format_exit_status (char *buffer, size_t buflen, int status)
+{
+  if (WIFEXITED(status))
+    {
+      if (WEXITSTATUS(status) == 0)
+	snprintf(buffer, buflen, _("Exited successfully"));
+      else
+	snprintf(buffer, buflen, _("Failed with status %d"),
+		 WEXITSTATUS(status));
+    }
+  else if (WIFSIGNALED(status)) 
+    snprintf(buffer, buflen,
+	     _("Terminated on signal %d"), WTERMSIG(status));
+  else if (WIFSTOPPED(status))
+    snprintf(buffer, buflen,
+	     _("Stopped on signal %d"), WSTOPSIG(status));
+#ifdef WCOREDUMP
+  else if (WCOREDUMP(status))
+    snprintf(buffer, buflen, _("Dumped core"));
+#endif
+  else
+    snprintf(buffer, buflen, _("Terminated"));
+  return buffer;
+}
+
+
 static RETSIGTYPE
 sig_cld (int code)
 {
   pid_t pid;
   int status;
-
+  char buffer[LINEBUFFER];
+  
   while ((pid = waitpid (-1, &status, WNOHANG)) > 0)
     {
       nchild--;
       info (VERBOSE,
 	    ngettext
-	    ("Child [%lu] finished. Exit status: %s. %d client left.",
-	     "Child [%lu] finished. Exit status: %s. %d clients left.",
+	    ("Child [%lu] finished. %s. %d client left.",
+	     "Child [%lu] finished. %s. %d clients left.",
 	     nchild), (unsigned long) pid,
-	    WIFEXITED (status) ? _("OK") : _("ERROR"), nchild);
+	    format_exit_status (buffer, sizeof buffer, status), nchild);
     }
+  signal (code, sig_cld);
   return;
 }
 
@@ -143,7 +172,7 @@ set_unprivileged_user (void)
 }
 
 int
-anubis_child_main (NET_STREAM * sd_client, struct sockaddr_in *addr)
+anubis_child_main (NET_STREAM *sd_client, struct sockaddr_in *addr)
 {
   int rc;
 
@@ -160,7 +189,6 @@ anubis_child_main (NET_STREAM * sd_client, struct sockaddr_in *addr)
 #else
   rc = anubis_transparent_mode (sd_client, addr);
 #endif /* WITH_GSASL */
-
   net_close_stream (sd_client);
   return rc;
 }
@@ -178,9 +206,6 @@ loop (int sd_bind)
 #ifdef USE_LIBWRAP
   struct request_info req;
 #endif /* USE_LIBWRAP */
-#ifdef HAVE_PAM
-  int pam_retval;
-#endif /* HAVE_PAM */
 
   addrlen = sizeof (addr);
   signal (SIGCHLD, sig_cld);
@@ -197,8 +222,7 @@ loop (int sd_bind)
 	    continue;
 	  else
 	    {
-	      anubis_error (SOFT,
-			    _("accept() failed: %s."), strerror (errno));
+	      anubis_error (0, errno, _("accept() failed"));
 	      continue;
 	    }
 	}
@@ -249,8 +273,7 @@ loop (int sd_bind)
 
 	  childpid = fork ();
 	  if (childpid == -1)
-	    anubis_error (HARD,
-			  _("daemon: Can't fork. %s."), strerror (errno));
+	    anubis_error (0, errno, _("daemon: cannot fork"));
 	  else if (childpid == 0)
 	    {			/* a child process */
 	      /* FIXME */
@@ -260,7 +283,6 @@ loop (int sd_bind)
 
 	  net_close_stream (&sd_client);
 	}
-      topt &= ~T_ERROR;
       cleanup_children ();
     }
   return;
@@ -332,10 +354,8 @@ stdinout (void)
   if (!(topt & T_LOCAL_MTA) && (strlen (session.mta) == 0))
     {
       options.termlevel = NORMAL;
-      anubis_error (HARD, _("The MTA has not been specified. "
-			    "Set the REMOTE-MTA or LOCAL-MTA."));
-      free_mem ();
-      return;
+      anubis_error (EXIT_FAILURE, 0, _("The MTA has not been specified. "
+			               "Set the REMOTE-MTA or LOCAL-MTA."));
     }
 
   net_create_stream (&sd_client, 0);
