@@ -28,6 +28,9 @@
 
 #ifdef HAVE_GPG
 #include <gpgme.h>
+#define obstack_chunk_alloc malloc
+#define obstack_chunk_free free
+#include <obstack.h>
 
 struct gpg_struct
 {
@@ -43,48 +46,75 @@ static int gpgme_init (void);
 static char *gpg_sign (char *);
 static char *gpg_encrypt (char *);
 static char *gpg_sign_encrypt (char *);
-static void gpgme_debug_info (GpgmeCtx);
+static void gpgme_debug_info (gpgme_ctx_t);
 
 #define EXTRA_GPG_BUF 4096
 #define fail_if_err(a) do { \
 		if (a) { \
-			anubis_error(HARD, _("GPGME: %s."), \
-			gpgme_strerror(a)); \
+			anubis_error(EXIT_FAILURE, 0, _("GPGME: %s."), \
+			             gpgme_strerror(a)); \
 		} \
 	} while(0)
 
-static void
-gpgme_debug_info (GpgmeCtx ctx)
+/* A replacement for the deprecated gpgme_data_rewind function. */
+static gpgme_error_t
+rewind_gpgme_data (gpgme_data_t dh)
 {
-  char *s = gpgme_get_op_info (ctx, 0);
-  if (s)
+  return (gpgme_data_seek (dh, 0, SEEK_SET) == -1)
+	   ? gpg_error_from_errno (errno) : 0;
+}
+
+
+static void
+gpgme_debug_info (gpgme_ctx_t ctx)
+{
+  /* FIXME: Current version of GPGMe does not provide any ways for
+     getting this info */
+}
+
+static void
+print_engine_info (gpgme_engine_info_t info)
+{
+  for (; info; info = info->next)
     {
-      fputs (s, stderr);
-      free (s);
+      fprintf (stderr, "Protocol: %s\n",
+	       gpgme_get_protocol_name (info->protocol));
+      fprintf (stderr, "Executable: %s\n",
+	       info->file_name ? info->file_name : "none");
+      fprintf (stderr, "Version: %s\n",
+	       info->version ? info->version : "N/A");
+      fprintf (stderr, "Required Version: %s\n",
+	       info->req_version ? info->req_version : "N/A");
     }
 }
 
-#define GPGME_REQ_VERSION "0.3.15"	/* GPGME 0.3.15 or later */
+#define GPGME_REQ_VERSION "0.9.0"	/* GPGME 0.9.0 or later */
 
 static int
 gpgme_init (void)
 {
-  GpgmeError err;
-
+  gpgme_error_t err;
+  
   if ((gpgme_check_version (GPGME_REQ_VERSION)) == 0)
     {
-      anubis_error (HARD, _("Install GPGME version %s or later."),
+      anubis_error (0, 0, _("Install GPGME version %s or later."),
 		    GPGME_REQ_VERSION);
       return -1;
     }
+
   if ((err = gpgme_engine_check_version (GPGME_PROTOCOL_OpenPGP)))
     {
-      anubis_error (HARD, _("GPGME: failed. %s."), gpgme_strerror (err));
+      anubis_error (0, 0, _("GPGME: failed. %s."), gpgme_strerror (err));
       return -1;
     }
+
   gpg.inited = 1;
   if (options.termlevel == DEBUG)
-    fputs (gpgme_get_engine_info (), stderr);
+    {
+      gpgme_engine_info_t info;
+      if (gpgme_get_engine_info (&info) == 0)
+	print_engine_info (info);
+    }
   return 0;
 }
 
@@ -99,10 +129,10 @@ passphrase_cb (void *hook, const char *desc, void **r_hd)
 static char *
 gpg_sign (char *gpg_data)
 {
-  GpgmeCtx ctx;
-  GpgmeError err = 0;
-  GpgmeData in, out;
-  GpgmeKey key;
+  gpgme_ctx_t ctx;
+  gpgme_error_t err = 0;
+  gpgme_data_t in, out;
+  gpgme_key_t key;
   char buf[256];
   char *p, *signed_data;
   int size;
@@ -124,36 +154,31 @@ gpg_sign (char *gpg_data)
 	      gpgme_key_release (key);
 	    }
 	}
-      if (err && err != GPGME_EOF)
+      if (err && gpg_err_code (err) != GPG_ERR_EOF)
 	{
-	  anubis_error (HARD, _("GPGME: Cannot list keys: %s"),
+	  anubis_error (0, 0, _("GPGME: Cannot list keys: %s"),
 			gpgme_strerror (err));
-	  topt |= T_ERROR;
+          xfree (signed_data);
+          gpgme_release (ctx);
+          return NULL;
 	}
     }
 
   p = getenv ("GPG_AGENT_INFO");
   if (!(p && strchr (p, ':')))
-    gpgme_set_passphrase_cb (ctx, (GpgmePassphraseCb) passphrase_cb, 0);
+    gpgme_set_passphrase_cb (ctx, (gpgme_passphrase_cb_t) passphrase_cb, 0);
   gpgme_set_textmode (ctx, 1);
   gpgme_set_armor (ctx, 1);
 
   fail_if_err (gpgme_data_new_from_mem (&in, gpg_data, strlen (gpg_data), 0));
   fail_if_err (gpgme_data_new (&out));
   fail_if_err (gpgme_op_sign (ctx, in, out, GPGME_SIG_MODE_CLEAR));
-  fail_if_err (gpgme_data_rewind (out));
+  fail_if_err (rewind_gpgme_data (out));
 
   if (options.termlevel == DEBUG)
     gpgme_debug_info (ctx);
 
-  if (topt & T_ERROR)
-    {
-      gpgme_release (ctx);
-      free (signed_data);
-      return 0;
-    }
-
-  while (!(err = gpgme_data_read (out, buf, sizeof (buf), &nread)))
+  while ((nread = gpgme_data_read (out, buf, sizeof (buf))) > 0)
     {
       if (size > nread)
 	{
@@ -170,8 +195,8 @@ gpg_sign (char *gpg_data)
 	}
       memset (buf, 0, sizeof (buf));
     }
-  if (err != GPGME_EOF)
-    fail_if_err (err);
+  if (nread == -1)
+    fail_if_err (errno);
 
   gpgme_data_release (in);
   gpgme_data_release (out);
@@ -179,61 +204,96 @@ gpg_sign (char *gpg_data)
   return signed_data;
 }
 
-static char *
-gpg_encrypt (char *gpg_data)
+static gpgme_key_t *
+create_key_array(gpgme_ctx_t ctx, struct obstack *stk)
 {
-  GpgmeCtx ctx;
-  GpgmeError err = 0;
-  GpgmeData in, out;
-  GpgmeRecipients rset;
+  gpgme_key_t tmpkey;
   char current_key[100];
-  char buf[256];
-  char *encrypted_data;
-  int i, j = 0;
-  int len, size;
-  size_t nread;
+  int i, j, len = strlen (gpg.encryption_keys);
 
-  size = strlen (gpg_data) + EXTRA_GPG_BUF;
-  encrypted_data = (char *) xmalloc (size);
-  memset (buf, 0, sizeof (buf));
   memset (current_key, 0, sizeof (current_key));
-
-  fail_if_err (gpgme_new (&ctx));
-  gpgme_set_armor (ctx, 1);
-
-  fail_if_err (gpgme_data_new_from_mem (&in, gpg_data, strlen (gpg_data), 0));
-  fail_if_err (gpgme_data_new (&out));
-  fail_if_err (gpgme_recipients_new (&rset));
-
-  len = strlen (gpg.encryption_keys);
-  for (i = 0; i <= len; i++)
+  for (i = j = 0; i <= len; i++)
     {
       if (gpg.encryption_keys[i] == ',' || gpg.encryption_keys[i] == '\0')
 	{
-	  fail_if_err (gpgme_recipients_add_name_with_validity (rset,
-								current_key,
-								GPGME_VALIDITY_FULL));
+	  gpgme_error_t err;
+	  current_key[j] = 0;
+	  err = gpgme_op_keylist_start (ctx, current_key, 0);
+	  while (!err)
+	    {
+	      err = gpgme_op_keylist_next (ctx, &tmpkey);
+	      if (err)
+		break;
+
+	      gpgme_get_key (ctx, current_key, &tmpkey, 0);
+	      obstack_grow (stk, &tmpkey, sizeof (tmpkey));
+	      if (options.termlevel == DEBUG)
+		{
+		  gpgme_user_id_t uid;
+
+		  for (uid = tmpkey->uids; uid; uid = uid->next)
+		    fprintf (stderr, "Using key %s: %s <%s>\n",
+			     uid->uid, uid->name, uid->email);
+		}
+	    }
+	  if (gpg_err_code (err) != GPG_ERR_EOF)
+	    {
+	      fprintf (stderr, "cannot list keys: %s\n",
+		       gpgme_strerror (err));
+	      exit (1);
+	    }
+	  
 	  memset (current_key, 0, sizeof (current_key));
 	  j = 0;
 	}
       else
 	current_key[j++] = gpg.encryption_keys[i];
     }
-  fail_if_err (gpgme_op_encrypt (ctx, rset, in, out));
-  fail_if_err (gpgme_data_rewind (out));
+
+  tmpkey = NULL;
+  obstack_grow (stk, &tmpkey, sizeof (tmpkey));
+  return obstack_finish (stk);
+}
+
+static char *
+gpg_encrypt (char *gpg_data)
+{
+  gpgme_ctx_t ctx;
+  gpgme_data_t in, out;
+  char buf[256];
+  char *encrypted_data;
+  size_t nread;
+  gpgme_key_t *keyptr;
+  struct obstack stk;
+  gpgme_encrypt_result_t result;
+  size_t size;
+  
+  size = strlen (gpg_data) + EXTRA_GPG_BUF;
+  encrypted_data = (char *) xmalloc (size);
+  memset (buf, 0, sizeof (buf));
+
+  fail_if_err (gpgme_new (&ctx));
+  gpgme_set_armor (ctx, 1);
+
+  fail_if_err (gpgme_data_new_from_mem (&in, gpg_data, strlen (gpg_data), 0));
+  fail_if_err (gpgme_data_new (&out));
+
+  obstack_init (&stk);
+  keyptr = create_key_array (ctx, &stk);
+  
+  fail_if_err (gpgme_op_encrypt (ctx, keyptr, GPGME_ENCRYPT_ALWAYS_TRUST,
+				 in, out));
+  result = gpgme_op_encrypt_result (ctx);
+  if (result->invalid_recipients)
+    anubis_error(0, 0, _("GPGME: Invalid recipient encountered: %s"),
+		 result->invalid_recipients->fpr);
+
+  fail_if_err (rewind_gpgme_data (out));
 
   if (options.termlevel == DEBUG)
     gpgme_debug_info (ctx);
 
-  if (topt & T_ERROR)
-    {
-      gpgme_recipients_release (rset);
-      gpgme_release (ctx);
-      free (encrypted_data);
-      return 0;
-    }
-
-  while (!(err = gpgme_data_read (out, buf, sizeof (buf), &nread)))
+  while ((nread = gpgme_data_read (out, buf, sizeof (buf))) > 0)
     {
       if (size > nread)
 	{
@@ -250,35 +310,84 @@ gpg_encrypt (char *gpg_data)
 	}
       memset (buf, 0, sizeof (buf));
     }
-  if (err != GPGME_EOF)
-    fail_if_err (err);
+  for (; *keyptr; keyptr++)
+    gpgme_key_unref (*keyptr);
+  obstack_free (&stk, NULL);
+
+  if (nread == -1)
+    fail_if_err (errno);
 
-  gpgme_recipients_release (rset);
   gpgme_data_release (in);
   gpgme_data_release (out);
   gpgme_release (ctx);
   return encrypted_data;
 }
 
+static int
+check_result (gpgme_sign_result_t result, gpgme_sig_mode_t type)
+{
+  int errcnt = 0;
+
+  if (result->invalid_signers)
+    {
+      anubis_error (0, 0, _("GPGME: Invalid signer found: %s"),
+  		    result->invalid_signers->fpr);
+      errcnt++;
+    } 
+
+  if (!result->signatures || result->signatures->next)
+    {
+      anubis_error (0, 0, _("GPGME: Unexpected number of signatures created"));
+      errcnt++;
+    }
+
+  if (result->signatures->type != type)
+    {
+      errcnt++;
+      anubis_error (0, 0, _("GPGME: Wrong type of signature created"));
+    }
+
+  if (result->signatures->pubkey_algo != GPGME_PK_DSA)
+    {
+      anubis_error (0, 0, _("GPGME: Wrong pubkey algorithm reported: %i"),
+		    result->signatures->pubkey_algo);
+      errcnt++;
+    }
+
+  if (result->signatures->hash_algo != GPGME_MD_SHA1)
+    {
+      anubis_error (0, 0, _("GPGME: Wrong hash algorithm reported: %i"),
+		    result->signatures->hash_algo);
+      errcnt++;
+    }
+
+  if (result->signatures->sig_class != 0)
+    {
+      anubis_error (0, 0, _("GPGME: Wrong signature class reported: %u"),
+ 		    result->signatures->sig_class);
+      errcnt++;
+    }
+  /* FIXME: fingerprint? */
+  return 0;
+}
+
 static char *
 gpg_sign_encrypt (char *gpg_data)
 {
-  GpgmeCtx ctx;
-  GpgmeError err = 0;
-  GpgmeData in, out;
-  GpgmeKey key;
-  GpgmeRecipients rset;
-  char current_key[100];
+  gpgme_ctx_t ctx;
+  gpgme_error_t err = 0;
+  gpgme_data_t in, out;
+  gpgme_key_t *keyptr, key;
   char buf[256];
   char *p, *se_data;		/* Signed-Encrypted Data */
-  int i, j = 0;
-  int len, size;
-  size_t nread;
-
+  int size;
+  gpgme_encrypt_result_t result;
+  gpgme_sign_result_t sign_result;
+  struct obstack stk;
+  
   size = strlen (gpg_data) + EXTRA_GPG_BUF;
   se_data = (char *) xmalloc (size);
   memset (buf, 0, sizeof (buf));
-  memset (current_key, 0, sizeof (current_key));
 
   fail_if_err (gpgme_new (&ctx));
 
@@ -293,72 +402,68 @@ gpg_sign_encrypt (char *gpg_data)
 	      gpgme_key_release (key);
 	    }
 	}
-      if (err && err != GPGME_EOF)
+      if (err && gpg_err_code (err) != GPG_ERR_EOF)
 	{
-	  anubis_error (HARD, _("GPGME: Cannot list keys: %s"),
+	  anubis_error (0, 0, _("GPGME: Cannot list keys: %s"),
 			gpgme_strerror (err));
-	  topt |= T_ERROR;
+          
+          gpgme_release (ctx);
+          xfree (se_data); 
+          return NULL;
 	}
     }
 
   p = getenv ("GPG_AGENT_INFO");
   if (!(p && strchr (p, ':')))
-    gpgme_set_passphrase_cb (ctx, (GpgmePassphraseCb) passphrase_cb, 0);
+    gpgme_set_passphrase_cb (ctx, (gpgme_passphrase_cb_t) passphrase_cb, 0);
   gpgme_set_armor (ctx, 1);
 
   fail_if_err (gpgme_data_new_from_mem (&in, gpg_data, strlen (gpg_data), 0));
   fail_if_err (gpgme_data_new (&out));
-  fail_if_err (gpgme_recipients_new (&rset));
 
-  len = strlen (gpg.encryption_keys);
-  for (i = 0; i <= len; i++)
+  obstack_init (&stk);
+  keyptr = create_key_array (ctx, &stk);
+  fail_if_err (gpgme_op_encrypt_sign (ctx, keyptr, GPGME_ENCRYPT_ALWAYS_TRUST,
+				      in, out));
+  result = gpgme_op_encrypt_result (ctx);
+  if (result->invalid_recipients)
+    anubis_error(0, 0, _("GPGME: Invalid recipient encountered: %s"),
+		 result->invalid_recipients->fpr);
+  sign_result = gpgme_op_sign_result (ctx);
+  if (check_result (sign_result, GPGME_SIG_MODE_NORMAL) == 0)
     {
-      if (gpg.encryption_keys[i] == ',' || gpg.encryption_keys[i] == '\0')
-	{
-	  fail_if_err (gpgme_recipients_add_name_with_validity (rset,
-								current_key,
-								GPGME_VALIDITY_FULL));
-	  memset (current_key, 0, sizeof (current_key));
-	  j = 0;
-	}
-      else
-	current_key[j++] = gpg.encryption_keys[i];
+      size_t nread;
+      fail_if_err (rewind_gpgme_data (out));
+
+      if (options.termlevel == DEBUG)
+        gpgme_debug_info (ctx);
+
+      while ((nread = gpgme_data_read (out, buf, sizeof (buf))) > 0)
+        {
+          if (size > nread)
+	    {
+	      strncat (se_data, buf, nread);
+	      size -= nread;
+ 	    }
+          else
+	    {
+	      size = EXTRA_GPG_BUF;
+	      se_data = (char *) xrealloc ((char *) se_data,
+				           strlen (se_data) + size);
+	      strncat (se_data, buf, nread);
+	      size -= nread;
+	    }
+          memset (buf, 0, sizeof (buf));
+        }
+     if (nread == -1)
+       fail_if_err (errno);
     }
-  fail_if_err (gpgme_op_encrypt_sign (ctx, rset, in, out));
-  fail_if_err (gpgme_data_rewind (out));
+  else 
+    xfree (se_data);
 
-  if (options.termlevel == DEBUG)
-    gpgme_debug_info (ctx);
-
-  if (topt & T_ERROR)
-    {
-      gpgme_recipients_release (rset);
-      gpgme_release (ctx);
-      free (se_data);
-      return 0;
-    }
-
-  while (!(err = gpgme_data_read (out, buf, sizeof (buf), &nread)))
-    {
-      if (size > nread)
-	{
-	  strncat (se_data, buf, nread);
-	  size -= nread;
-	}
-      else
-	{
-	  size = EXTRA_GPG_BUF;
-	  se_data = (char *) xrealloc ((char *) se_data,
-				       strlen (se_data) + size);
-	  strncat (se_data, buf, nread);
-	  size -= nread;
-	}
-      memset (buf, 0, sizeof (buf));
-    }
-  if (err != GPGME_EOF)
-    fail_if_err (err);
-
-  gpgme_recipients_release (rset);
+  for (; *keyptr; keyptr++)
+    gpgme_key_unref (*keyptr);
+  obstack_free (&stk, NULL);
   gpgme_data_release (in);
   gpgme_data_release (out);
   gpgme_release (ctx);
@@ -379,10 +484,6 @@ gpg_proc (MESSAGE * msg, char *(*proc) (char *input))
 #endif /* HAVE_SETENV or HAVE_PUTENV */
 
   buf = proc (msg->body);
-
-  if (topt & T_ERROR)
-    return;
-
   xfree (msg->body);
   msg->body = buf;
 
