@@ -179,6 +179,7 @@
 #define DEFAULT_LOCAL_RCFILE ".anubisrc"
 #define DEFAULT_SSL_PEM "anubis.pem"
 #define DEFAULT_SIGFILE ".signature"
+#define DEFAULT_PIDFILE "anubis.pid"
 #define BEGIN_TRIGGER "@@"
 #define LF "\n"
 #define CRLF "\r\n"
@@ -210,11 +211,6 @@
 #define FAILS 1
 #define ALL   2
 
-/* error types */
-#define SOFT   1
-#define HARD   2
-#define SYNTAX 3
-
 typedef enum anubis_mode
 {
   anubis_transparent,
@@ -223,7 +219,7 @@ typedef enum anubis_mode
 ANUBIS_MODE;
 
 /* bit values for topt */
-#define T_ERROR             0x00000001
+#define T_DISABLE_SYSLOG    0x00000001 /* Used to be T_ERROR */
 #define T_SOCKS             0x00000002
 #define T_SOCKS_V4          0x00000004
 #define T_SOCKS_AUTH        0x00000008
@@ -330,9 +326,12 @@ int check_superuser (void);
 int check_username (char *);
 int check_filemode (char *);
 int check_filename (char *, time_t *);
+void write_pid_file (void);
 
 /* errs.c */
-void anubis_error (int, const char *, ...);
+#define EXIT_ABORT 256
+void anubis_error (int, int, const char *, ...);
+void anubis_warning (int error_code, const char *fmt, ...);
 void socket_error (const char *);
 void hostname_error (char *);
 
@@ -370,6 +369,7 @@ void translate_section_init (void);
 /* tunnel.c */
 void smtp_session (void);
 void smtp_session_transparent (void);
+void set_ehlo_domain (char *domain);
 
 /* message.c */
 void message_add_body (MESSAGE *, char *, char *);
@@ -579,4 +579,11 @@ int xdatabase (char *command);
 void xdatabase_capability (char *reply, size_t reply_size);
 void xdatabase_enable (void);
 
+/* md5.c */
+int anubis_md5_file (unsigned char *digest, int fd);
+void string_bin_to_hex (unsigned char *output, unsigned char *input, int inlen);
+int string_hex_to_bin (unsigned char *output, unsigned char *input, int inlen);
+
+#define MD5_DIGEST_BYTES 16
+
 /* EOF */
