@@ -118,9 +118,7 @@ file_id_add (char *filename)
   struct file_id *fid;
   if (stat (filename, &st))
     {
-      anubis_error (SOFT,
-		    _("cannot stat file `%s': %s"), filename,
-		    strerror (errno));
+      anubis_error (0, errno, _("cannot stat file `%s'"), filename);
       return -1;
     }
   fid = xmalloc (sizeof (*fid));
@@ -327,6 +325,15 @@ control_parser (int method, int key, ANUBIS_LIST * arglist,
 	xfree (options.ulogfile);
 	options.ulogfile = allocbuf (arg, MAXPATHLEN);
       }
+    else if (getpid () == 0)
+      anubis_warning (0,
+	       _("`logfile' directive is ignored in main configuration file"));
+    else
+      {
+	topt |= T_DISABLE_SYSLOG;
+	xfree (options.ulogfile);
+	options.ulogfile = allocbuf (arg, MAXPATHLEN);
+      }
     break;
 
   case KW_LOGLEVEL:
@@ -468,6 +475,8 @@ static struct rc_kwdef init_supervisor_kw[] = {
   { "drop-unknown-user",  KW_DROP_UNKNOWN_USER },
   { "rule-priority",      KW_RULE_PRIORITY },
   { "control-priority",   KW_CONTROL_PRIORITY },
+  { "logfile",            KW_LOGFILE },
+  { "loglevel",           KW_LOGLEVEL },
   { NULL }
 };
 
@@ -697,7 +706,7 @@ rcfile_call_section (int method, char *name, void *data, MESSAGE * msg)
 {
   RC_SECTION *sec = rc_section_lookup (parse_tree, name);
   if (!sec)
-    anubis_error (SOFT, _("No such section: %s"), name);
+    anubis_error (0, 0, _("No such section: %s"), name);
   rc_call_section (method, sec, anubis_rc_sections, data, msg);
 }
 
