@@ -356,9 +356,6 @@ smtp_session_transparent (void)
       if (process_command (&msg, command))
 	continue;
 
-      if (topt & T_ERROR)
-	break;
-
       if (transfer_command (&msg, command) == 0)
 	break;
     }
@@ -396,9 +393,6 @@ smtp_session (void)
       if (process_command (&msg, command))
 	continue;
 
-      if (topt & T_ERROR)
-	break;
-
       if (transfer_command (&msg, command) == 0)
 	break;
     }
@@ -451,6 +445,8 @@ static int
 handle_starttls (char *command)
 {
 #ifdef USE_SSL
+  NET_STREAM stream;
+  
   if (topt & T_SSL_FINISHED)
     {
       if (topt & T_SSL_ONEWAY)
@@ -474,6 +470,7 @@ handle_starttls (char *command)
 
   if (!(topt & T_LOCAL_MTA))
     {
+      NET_STREAM stream;
       char reply[LINEBUFFER + 1];
       swrite (CLIENT, remote_server, "STARTTLS" CRLF);
 
@@ -484,18 +481,18 @@ handle_starttls (char *command)
 	{
 	  remcrlf (reply);
 	  info (VERBOSE, _("WARNING: %s"), reply);
-	  anubis_error (HARD, _("STARTTLS command failed."));
+	  anubis_error (0, 0, _("STARTTLS command failed."));
 	  return 0;
 	}
 
-      secure.client = start_ssl_client (remote_server,
-					secure.cafile,
-					options.termlevel > NORMAL);
-      if (!secure.client || (topt & T_ERROR))
+      stream = start_ssl_client (remote_server,
+				 secure.cafile,
+				 options.termlevel > NORMAL);
+      if (!stream)
 	return 0;
-      remote_server = (void *) secure.client;
+      remote_server = stream;
     }
-
+  
   /*
      Make the TLS/SSL connection with SMTP client
      (client connected with the Tunnel).
@@ -528,16 +525,16 @@ handle_starttls (char *command)
     check_filemode (secure.key);
 
   swrite (SERVER, remote_client, "220 2.0.0 Ready to start TLS" CRLF);
-  secure.server = start_ssl_server (remote_client,
-				    secure.cafile,
-				    secure.cert,
-				    secure.key, options.termlevel > NORMAL);
-  if (!secure.server || (topt & T_ERROR))
+  stream = start_ssl_server (remote_client,
+			     secure.cafile,
+			     secure.cert,
+			     secure.key, options.termlevel > NORMAL);
+  if (!stream)
     {
       swrite (SERVER, remote_client, "454 4.3.3 TLS not available" CRLF);
       return 0;
     }
-  remote_client = secure.server;
+  remote_client = stream;
   topt |= T_SSL_FINISHED;
 #else
   swrite (SERVER, remote_client, "503 5.5.0 TLS not available" CRLF);
@@ -559,13 +556,20 @@ process_command (MESSAGE * msg, char *command)
   if (!strncmp (buf, "starttls", 8))
     return handle_starttls (buf);
   else if (!strncmp (buf, "xdatabase", 9))
-    return xdatabase (buf);
+    return xdatabase (buf + 9);
 
   return 0;
 }
 
+void
+set_ehlo_domain (char *domain)
+{
+  xfree (smtp_ehlo_domain_name);
+  smtp_ehlo_domain_name = strdup (domain);
+}
+
 static void
-save_ehlo_domain(char *command)
+save_ehlo_domain (char *command)
 {
   char *p, *endp;
   
@@ -577,8 +581,7 @@ save_ehlo_domain(char *command)
   for (endp = p + strlen (p) - 1; endp >= p && isspace(*endp); endp--)
     ;
   endp[1] = 0;
-  xfree(smtp_ehlo_domain_name);
-  smtp_ehlo_domain_name = strdup(p);
+  set_ehlo_domain (p);
 }
 
 static int
@@ -599,6 +602,7 @@ handle_ehlo (char *command, char *reply, size_t reply_size)
   if ((topt & T_SSL_ONEWAY)
       && (topt & T_STARTTLS) && !(topt & T_SSL_FINISHED))
     {
+      NET_STREAM stream;
       char ehlo[128];
       char newreply[LINEBUFFER + 1];
 
@@ -606,6 +610,8 @@ handle_ehlo (char *command, char *reply, size_t reply_size)
          The 'ONEWAY' method is used when your MUA doesn't
          support the TLS/SSL, but your MTA does.
          Make the TLS/SSL connection with ESMTP server.
+	 FIXME: The diagnostic message below is not correct in
+	 authmode.
        */
 
       info (NORMAL,
@@ -619,24 +625,23 @@ handle_ehlo (char *command, char *reply, size_t reply_size)
 	{
 	  remcrlf (newreply);
 	  info (VERBOSE, _("WARNING: %s"), newreply);
-	  anubis_error (SOFT, _("STARTTLS (ONEWAY) command failed."));
+	  anubis_error (0, 0, _("STARTTLS (ONEWAY) command failed."));
 	  topt &= ~T_SSL_ONEWAY;
 	  swrite (SERVER, remote_client, reply);
 	  return 1;
 	}
 
-      secure.client = start_ssl_client (remote_server,
-					secure.cafile,
-					options.termlevel > NORMAL);
-      if (!secure.client || (topt & T_ERROR))
+      stream = start_ssl_client (remote_server,
+				 secure.cafile,
+				 options.termlevel > NORMAL);
+      if (!stream)
 	{
-	  topt &= ~T_ERROR;
 	  topt &= ~T_SSL_ONEWAY;
 	  swrite (SERVER, remote_client, reply);
 	  return 1;
 	}
 
-      remote_server = (void *) secure.client;
+      remote_server = stream;
       topt |= T_SSL_FINISHED;
 
       /*
@@ -688,8 +693,6 @@ transfer_command (MESSAGE * msg, char *command)
   safe_strcpy (buf, command);
   make_lowercase (buf);
   swrite (CLIENT, remote_server, command);
-  if (topt & T_ERROR)
-    return 0;
 
   if (!strncmp (buf, "ehlo", 4))
     {
@@ -700,8 +703,6 @@ transfer_command (MESSAGE * msg, char *command)
     get_response_smtp (CLIENT, remote_server, reply, sizeof (reply));
 
   swrite (SERVER, remote_client, reply);
-  if (topt & T_ERROR)
-    return 0;
 
   if (isdigit ((unsigned char) reply[0]) && (unsigned char) reply[0] < '4')
     {
@@ -711,12 +712,10 @@ transfer_command (MESSAGE * msg, char *command)
 	{
 	  message_free (msg);
 	  message_init (msg);
-	  topt &= ~T_ERROR;
 	}
       else if (strncmp (buf, "data", 4) == 0)
 	{
 	  process_data (msg);
-	  topt &= ~T_ERROR;
 	}
     }
   return 1;			/* OK */
@@ -787,3 +786,4 @@ transfer_body (MESSAGE * msg)
 }
 
 /* EOF */
+
